import { Validators } from '@angular/forms';
import { uniq, lazy, traverseControl } from 'ngx-reactive-form/utils';

class DependentValidatorBuilderImpl {
    constructor(affectedControls, validate) {
        this.affectedControls = affectedControls;
        this.validate = validate;
    }
    inverse() {
        return new DependentValidatorBuilderImpl(this.affectedControls, () => !this.validate());
    }
}

class DependentValidatorBuilderRootImpl {
    constructor(form) {
        this.form = form;
    }
    all(...builders) {
        return new DependentValidatorBuilderImpl(() => uniq([].concat(builders.map(b => b.affectedControls()))), () => builders.every(b => b.validate()));
    }
    any(...builders) {
        return new DependentValidatorBuilderImpl(() => uniq([].concat(builders.map(b => b.affectedControls()))), () => builders.some(b => b.validate()));
    }
    validator(predicate, firstPath, ...paths) {
        const formFn = lazy(() => this.form());
        const ctrlFn = lazy(() => {
            if (typeof firstPath === 'string' || typeof firstPath === 'number') {
                return traverseControl(formFn(), firstPath, ...paths);
            }
            else if (typeof firstPath === 'function') {
                return firstPath(formFn());
            }
        });
        return new DependentValidatorBuilderImpl(() => [ctrlFn()], () => {
            const ctrl = ctrlFn();
            return ctrl && typeof predicate === 'function' && predicate(ctrl.value, formFn());
        });
    }
}

class DependentValidatorImpl {
    constructor(rootFn, builderFn) {
        this.rootFn = rootFn;
        this.builderFn = builderFn;
        this.builder = lazy(() => {
            if (typeof this.builderFn === 'function') {
                const builderRoot = new DependentValidatorBuilderRootImpl(() => this.rootFn());
                return this.builderFn(builderRoot);
            }
        });
        this.subscribedControls = [];
        this.subscription = lazy(() => this.builder().affectedControls()
            .filter(item => !!item).map(item => item
            .valueChanges.subscribe(() => this.subscribedControls
            .forEach(control => control.updateValueAndValidity()))), subs => subs.forEach(s => s.unsubscribe()));
    }
    reset() {
        this.builder.reset();
        this.subscription.reset();
    }
    applyValidator(path, ...validators) {
        const paths = Array.isArray(path) ? path : [path];
        const ctrl = traverseControl(this.rootFn(), ...paths);
        if (ctrl) {
            ctrl.setValidators(this.createValidator(...validators));
        }
        return this;
    }
    createValidator(...validators) {
        const validate = Validators.compose(validators);
        let ctrlSubscribed = false;
        return ctrl => {
            let builder;
            if (!ctrlSubscribed) {
                ctrlSubscribed = true;
                if (this.subscribedControls.indexOf(ctrl) === -1) {
                    this.subscribedControls.push(ctrl);
                }
            }
            if (!this.subscription.isInit() && this.rootFn()) {
                builder = this.builder();
                this.subscription();
            }
            else if (this.builder.isInit()) {
                builder = this.builder();
            }
            return (!builder || builder.validate()) ? validate(ctrl) : null;
        };
    }
}

/**
 * Generated bundle index. Do not edit.
 */

export { DependentValidatorImpl };
//# sourceMappingURL=ngx-reactive-form-dependant-validator.js.map
