export * from 'ngx-reactive-form/declarations';
import { __classPrivateFieldGet } from 'tslib';
import * as i0 from '@angular/core';
import { InjectFlags, Injectable, Injector } from '@angular/core';
import { FormGroup, FormArray, NgControl, FormBuilder } from '@angular/forms';
import { DependentValidatorImpl } from 'ngx-reactive-form/dependant-validator';
import { lazy, compositeValidator, compositeAsyncValidator, tapControls } from 'ngx-reactive-form/utils';
export { compositeAsyncValidator, compositeValidator, tapControls, traverseControl } from 'ngx-reactive-form/utils';
import { Subscription } from 'rxjs';
import { map, debounceTime } from 'rxjs/operators';

var _FormUtils_root, _FormUtils_compositeValidator, _FormUtils_compositeAsyncValidator;
class FormUtils {
    constructor(injector, rootFn) {
        this.injector = injector;
        this.rootFn = rootFn;
        _FormUtils_root.set(this, lazy(() => {
            if (this.rootFn instanceof FormGroup || this.rootFn instanceof FormArray) {
                return this.rootFn;
            }
            else if (typeof this.rootFn === 'function') {
                return this.rootFn();
            }
        }));
        _FormUtils_compositeValidator.set(this, lazy(() => {
            const v = compositeValidator();
            return () => v(this.root);
        }));
        _FormUtils_compositeAsyncValidator.set(this, lazy(() => {
            const v = compositeAsyncValidator();
            return () => v(this.root);
        }));
    }
    get root() {
        const r = __classPrivateFieldGet(this, _FormUtils_root, "f");
        if (!r()) {
            r.reset();
        }
        return r();
    }
    get compositeValidator() { return __classPrivateFieldGet(this, _FormUtils_compositeValidator, "f").call(this); }
    get compositeAsyncValidator() { return __classPrivateFieldGet(this, _FormUtils_compositeAsyncValidator, "f").call(this); }
    tapControls(action, updateState = true) {
        tapControls(this.root, action, updateState);
    }
    createDependentValidator(builderFn) {
        return new DependentValidatorImpl(() => this.root, builderFn);
    }
    makePristine() {
        this.tapControls(ctrl => {
            ctrl.markAsPristine();
            ctrl.markAsUntouched();
        });
    }
    makeDirty() {
        this.tapControls(ctrl => {
            ctrl.markAsDirty();
            ctrl.markAsTouched();
        });
    }
    /**
     * Consume this method on **NG_VALUE_ACCESSOR** `afterViewInit()`
     * ### Note
     * Add `ExtendedFormBuilder` to component providers
     */
    registerSubFormValidator() {
        var _a, _b, _c;
        const subscriptions = [];
        const ctrl = (_a = this.injector.get(NgControl, null, InjectFlags.Self)) === null || _a === void 0 ? void 0 : _a.control;
        if (ctrl) {
            (_b = ctrl.validator) !== null && _b !== void 0 ? _b : (ctrl.validator = this.compositeValidator);
            (_c = ctrl.asyncValidator) !== null && _c !== void 0 ? _c : (ctrl.asyncValidator = this.compositeAsyncValidator);
            subscriptions.push(ctrl.statusChanges.pipe(map(() => ctrl.invalid), debounceTime(50)).subscribe(invalid => {
                this.root.updateValueAndValidity({ emitEvent: true });
                if (invalid && !ctrl.pristine) {
                    this.makeDirty();
                }
            }));
        }
        return new Subscription(() => subscriptions.forEach(s => s.unsubscribe()));
    }
}
_FormUtils_root = new WeakMap(), _FormUtils_compositeValidator = new WeakMap(), _FormUtils_compositeAsyncValidator = new WeakMap();

class ExtendedFormBuilderResult {
    constructor(injector, factory) {
        this.injector = injector;
        this.factory = factory;
        this.fb = this.injector.get(FormBuilder);
        this.validators = [];
        this._formUtils = lazy(() => new FormUtils(this.injector, () => this.form));
        this.form = this.factory(this.fb, dv => {
            const v = this.formUtils.createDependentValidator(dv);
            this.validators.push(v);
            return v;
        });
    }
    get formUtils() {
        return this._formUtils();
    }
    updateValidators() {
        this.validators.forEach(item => item.reset());
        this.formUtils.tapControls();
    }
}
class ExtendedFormBuilder {
    constructor(injector) {
        this.injector = injector;
    }
    build(factory) {
        return new ExtendedFormBuilderResult(this.injector, factory);
    }
}
ExtendedFormBuilder.ɵprov = i0.ɵɵdefineInjectable({ factory: function ExtendedFormBuilder_Factory() { return new ExtendedFormBuilder(i0.ɵɵinject(i0.INJECTOR)); }, token: ExtendedFormBuilder, providedIn: "root" });
ExtendedFormBuilder.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
ExtendedFormBuilder.ctorParameters = () => [
    { type: Injector }
];

/**
 * Generated bundle index. Do not edit.
 */

export { ExtendedFormBuilder, FormUtils };
//# sourceMappingURL=ngx-reactive-form.js.map
